@@ -92,18 +92,24 @@ func newFrame(anc *frame, length int, id uint64) *frame {
 
 func (f *frame) runid() uint64      { return atomic.LoadUint64(&f.id) }
 func (f *frame) setrunid(id uint64) { atomic.StoreUint64(&f.id, id) }
-func (f *frame) clone() *frame {
+func (f *frame) clone(fork bool) *frame {
 	f.mutex.RLock()
 	defer f.mutex.RUnlock()
-	return &frame{
+	nf := &frame{
 		anc:       f.anc,
 		root:      f.root,
-		data:      f.data,
 		deferred:  f.deferred,
 		recovered: f.recovered,
 		id:        f.runid(),
 		done:      f.done,
 	}
+	if fork {
+		nf.data = make([]reflect.Value, len(f.data))
+		copy(nf.data, f.data)
+	} else {
+		nf.data = f.data
+	}
+	return nf
 }
 
 // Exports stores the map of binary packages per package path.
@@ -208,7 +214,7 @@ type Panic struct {
 }
 
 // TODO: Capture interpreter stack frames also and remove
-// fmt.Println(n.cfgErrorf("panic")) in runCfg.
+// fmt.Fprintln(n.interp.stderr, oNode.cfgErrorf("panic")) in runCfg.
 
 func (e Panic) Error() string { return fmt.Sprint(e.Value) }
 
